@@ -9,9 +9,13 @@ from io import IOBase
 from typing import Union, List, Optional, Dict, DefaultDict, Tuple, BinaryIO, Any
 from urllib.parse import urlencode, urljoin
 
+from requests import PreparedRequest
 from requests.auth import HTTPBasicAuth, HTTPDigestAuth, AuthBase
+from requests.structures import CaseInsensitiveDict
 
-from .utils import get_real_file_path, triple_or_double_tostring, APPLICATION_JSON
+from .utils import get_real_file_path, triple_or_double_tostring, APPLICATION_JSON, json_to_urlencoded_array
+
+BASEIC_AUTHORIZATION_HEADER = "Authorization"
 
 try:
     import jstyleson as json
@@ -42,6 +46,8 @@ MIME_TYPE_JSON = "application/json"
 FORM_URLENCODED = "application/x-www-form-urlencoded"
 MULTIPART_FORM_INPUT = "multipart/form-data"
 
+CONTENT_TYPE = 'content-type'
+
 dothttp_model = metamodel_from_file(get_real_file_path(path="http.tx", current_file=__file__))
 
 
@@ -68,7 +74,7 @@ class Config:
 
 @dataclass
 class Payload:
-    data: Optional[Union[str, bytes, Dict]] = None
+    data: Optional[Union[str, bytes, Dict, BinaryIO]] = None
     json: Optional[Dict] = None
     header: Optional[str] = None
     filename: str = None
@@ -96,6 +102,25 @@ class HttpDef:
     test_script: str = ""
 
     def get_har(self):
+        if self.auth:
+            request = self.get_prepared_request()
+            # till now url and headers are enough
+            # in future according that,
+            # include all params
+            # not giving inaccurate results
+            self.auth(request)
+            # For any other auth, it is bad
+            if isinstance(self.auth, HTTPBasicAuth):
+                self.headers[BASEIC_AUTHORIZATION_HEADER] = request.headers.get(BASEIC_AUTHORIZATION_HEADER)
+            # # har with httpdigest is not possible
+            # # as auth is set once, request is redirected to 401
+            # # for now, ignoring
+            # # also for har, certificate also has to be ignored
+            # # har does't support this
+            # # --------------------------
+            # elif isinstance(self.auth, HTTPDigestAuth):
+            #     self.headers[BASEIC_AUTHORIZATION_HEADER] = self.auth.build_digest_header(self.method, self.url)
+
         target = {
             "url": self.url,
             "method": self.method,
@@ -105,6 +130,22 @@ class HttpDef:
         }
         return target
 
+    def get_prepared_request(self):
+        prep = PreparedRequest()
+        prep.prepare_url(self.url, self.query)
+        prep.prepare_method(self.method)
+        prep.prepare_headers(self.headers)
+        prep.prepare_auth(self.auth, self.url)
+        payload = self.payload
+        prep.prepare_body(data=payload.data, json=payload.json, files=payload.files)
+        # prep.prepare_hooks({"response": self.save_cookie_call_back})
+        if payload.header and CONTENT_TYPE not in prep.headers:
+            # if content-type is provided by header
+            # we will not wish to update it
+            prep.headers[CONTENT_TYPE] = payload.header
+        request_logger.debug(f'request prepared completely {prep}')
+        return prep
+
     def get_payload(self):
         if not self.payload:
             return None
@@ -113,10 +154,19 @@ class HttpDef:
         if payload.data:
             if isinstance(payload.data, dict):
                 return_data["mimeType"] = FORM_URLENCODED
-                return_data["text"] = urlencode(payload.data)
+                return_data["text"] = urlencode(json_to_urlencoded_array(payload.data))
             else:
                 return_data["mimeType"] = payload.header or "text/plain"
-                return_data["text"] = payload.data
+                if isinstance(payload.data, (str, bytes)):
+                    return_data["text"] = payload.data
+                else:
+                    try:
+                        data_read = payload.data.read()
+                        return_data["text"] = data_read.decode("utf-8")
+                    except:
+                        return_data["text"] = "file conversion to uft-8 ran into error"
+                    finally:
+                        payload.data.close()
         elif payload.json:
             return_data["mimeType"] = APPLICATION_JSON
             return_data["text"] = json.dumps(payload.json)
@@ -337,7 +387,7 @@ class BaseModelProcessor:
     def load_content(self):
         if not os.path.exists(self.file):
             raise HttpFileNotFoundException(file=self.file)
-        with open(self.file, 'r') as f:
+        with open(self.file, 'r', encoding="utf-8") as f:
             self.original_content = self.content = f.read()
 
     def get_updated_content(self, content) -> str:
@@ -425,7 +475,9 @@ class HttpDefBase(BaseModelProcessor):
                 4. dev can define in data/file/files's type attribute section for ('content-type')
         :return:
         """
-        headers = {}
+        ## headers are case insensitive
+        ## having duplicate headers creates problem while exporting to curl,postman import..
+        headers = CaseInsensitiveDict()
         headers.update(self.default_headers)
         self.load_headers_to_dict(self.base_http, headers)
         self.load_headers_to_dict(self.http, headers)
@@ -522,16 +574,7 @@ class HttpDefBase(BaseModelProcessor):
             # varstring hanlding
             return Payload(data=d, header=FORM_URLENCODED)
         elif upload_filename := self.http.payload.file:
-            upload_filename = self.get_updated_content(upload_filename)
-            request_logger.debug(
-                f'payload will be loaded from `{upload_filename}`')
-            if not os.path.exists(upload_filename):
-                request_logger.debug(
-                    f'payload file `{upload_filename}` Not found. ')
-                raise DataFileNotFoundException(datafile=upload_filename)
-            mimetype = self.get_mimetype_from_file(upload_filename, self.http.payload.type)
-            with open(upload_filename, 'rb') as f:
-                return Payload(data=f.read(), header=mimetype, filename=upload_filename)
+            return self.load_payload_fileinput(upload_filename)
         elif json_data := self.http.payload.json:
             d = json_or_array_to_json(json_data, self.get_updated_content)
             return Payload(json=d, header=MIME_TYPE_JSON)
@@ -552,6 +595,18 @@ class HttpDefBase(BaseModelProcessor):
             return Payload(files=files, header=MULTIPART_FORM_INPUT)
         return Payload()
 
+    def load_payload_fileinput(self, upload_filename):
+        upload_filename = self.get_updated_content(upload_filename)
+        request_logger.debug(
+            f'payload will be loaded from `{upload_filename}`')
+        if not os.path.exists(upload_filename):
+            request_logger.debug(
+                f'payload file `{upload_filename}` Not found. ')
+            raise DataFileNotFoundException(datafile=upload_filename)
+        mimetype = self.get_mimetype_from_file(upload_filename, self.http.payload.type)
+        f = open(upload_filename, 'rb')
+        return Payload(data=f, header=mimetype, filename=upload_filename)
+
     @staticmethod
     def get_mimetype_from_file(filename, mimetype: Optional[str]) -> Optional[str]:
         if mimetype:
